@@ -2,6 +2,7 @@
 #define EVALUATE_H
 
 struct expression;
+struct expression_list;
 struct statement;
 struct symbol;
 struct symbol_list;
@@ -25,4 +26,10 @@ struct symbol *evaluate_statement(struct statement *stmt);
 // @list: the list of the symbol to be evaluated
 void evaluate_symbol_list(struct symbol_list *list);
 
+///
+// evaluate the arguments of a function
+// @argtypes: the list of the types in the prototype
+// @args: the list of the effective arguments
+int evaluate_arguments(struct symbol_list *argtypes, struct expression_list *args);
+
 #endif
