@@ -18,13 +18,6 @@ int foo(int a, int b, int p)
  * check-name: call-inlined
  * check-command: test-linearize -Wno-decl $file
  *
- * check-output-start
-foo:
-.L0:
-	<entry-point>
-	select.32   %r10 <- %arg3, %arg3, $0
-	ret.32      %r10
-
-
- * check-output-end
+ * check-output-ignore
+ * check-output-returns: %arg3
  */
